# oplib.pl - Subroutines to modify program options
# This file is part of WebRSH 1.1b (beta)
# Copyright (C) 1997,1998  Yoram Last (ylast@mindless.com)
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

sub DoOpCom { # Handles option setting commands.

  local($opt,$opcom) = @_;
  $OpType = $opt;
  $PrintHelp = 0;
  $PrOpMenu = 0;
  local($ScriptFileName) = $ENV{'SCRIPT_FILENAME'};
  $ScriptFileName = $0 unless ($ScriptFileName);
  if ($opt) {
    $opcom =~ s/\s//g;
    if ($opcom eq 'OK') {
      if ($opt eq 'mime') {
        &ModMime($input{'mtable'});
      } else {
        &ModOptions($ScriptFileName,$opt);
      };
    } elsif ($opcom eq 'Cancel') {
    } elsif ($opcom eq 'Apply') {
      if ($opt eq 'mime') {
        &ModMime($input{'mtable'});
      } else {
        &ModOptions($ScriptFileName,$opt);
      };
      $PrOpMenu = 1;
    } elsif ($opcom eq 'Help') {
      $PrintHelp = 1;
      $PrOpMenu = 1;
    } elsif (!$opcom) {
      $PrOpMenu = 1;
    } else {
      &WebrshDie("Unknown command\: $opcom");
    };
  };
  $output = 'ok' unless ($output);
}

sub ModOptions { # Modifies WebRSH options.

  local($filename,$opt) = @_;
  local($mod,$buf,$file,$prestr,$varstr,$remstr) = (0,'','','','','','');
  open(FILE,"<$filename") || &WebrshDie("Can\'t open $filename\: $!");
  while (read(FILE,$buf,$BufSize)) {
    $file .= $buf; # join('',$file,$buf);
  };
  close FILE;
  $file =~ m/^([\x00-\x7f]+\n\#\-\!\-\!\-\!\>)([\x00-\x7f]+)(\n\#\-\!\-\!\-\!\>[\x00-\x7f]+)$/
    || &WebrshDie("$filename appears to be corrupted");
  $prestr = $1; $varstr = $2; $remstr = $3;
  if ($opt eq 'editor') {
    ($varstr,$mod) = &ModEdOptions($filename,$varstr);
  } elsif ($opt eq 'general') {
    ($varstr,$mod) = &ModGenOptions($filename,$varstr);
  } elsif ($opt eq 'access') {
    ($varstr,$mod) = &ModAccOptions($filename,$varstr);
  } elsif ($opt eq 'dirs') {
    ($varstr,$mod) = &ModDirOptions($filename,$varstr);
  };
  if ($mod) {
    local($backfile,$backrem) = ('','');
    if ($MakeBack) {
      $filename =~ m/^(.+)(\.[^\.]+)$/ || &WebrshDie("Can\'t process extensionless files");
      $backfile = "$1\-$2";
      &GetUmask if ($OStype eq 'U');
      open(FILE,">$backfile") || &WebrshDie("Can\'t write to $backfile\: $!");
      print FILE $file;
      close FILE;
      $backrem = "Backup copy had been saved as $backfile\.";
    };
    open(FILE,">$filename") || &WebrshDie("Can\'t write to $filename\: $!",$backrem);
    print FILE $prestr,$varstr,$remstr;
    close FILE;
    $output = join("\n","$filename  had been modified\.",$backrem);
  };
}

sub ModEdOptions { # Modifies Text Editor options.

  local($filename,$varstr) = @_;
  local($mod) = 0;
  local($tEdCols) = $input{'edcols'};
  $tEdCols =~ s/[^\d]//g;
  if ($tEdCols ne $EdCols) {
    $mod = 1;
    $EdCols = $tEdCols;
    $varstr =~ s/(\n\$EdCols\s+\=\s+\')\d*\'\;/$1$EdCols\'\;/ || 
      &WebrshDie("$filename appears to be corrupted");
  };
  local($tEdRows) = $input{'edrows'};
  $tEdRows =~ s/[^\d]//g;
  if ($tEdRows ne $EdRows) {
    $mod = 1;
    $EdRows = $tEdRows;
    $varstr =~ s/(\n\$EdRows\s+\=\s+\')\d*\'\;/$1$EdRows\'\;/ || 
      &WebrshDie("$filename appears to be corrupted");
  };
  local($tEdWrap) = $input{'edwrap'};
  $tEdWrap =~ s/[^\w]//g;
  if ($tEdWrap ne $EdWrap) {
    $mod = 1;
    $EdWrap = $tEdWrap;
    $varstr =~ s/(\n\$EdWrap\s+\=\s+\')\w*\'\;/$1$EdWrap\'\;/ || 
      &WebrshDie("$filename appears to be corrupted");
  };
  local($tEdEmptyLnHldr) = $input{'lnhldr'};
  if ($tEdEmptyLnHldr) {
    $tEdEmptyLnHldr =~ m/\'(.*)\'/ || 
      &WebrshDie("Bad Place Holder for Empty Lines");
    $tEdEmptyLnHldr = $1;
  };
  if (!$tEdEmptyLnHldr) {$tEdEmptyLnHldr = '';};
  if ($tEdEmptyLnHldr ne $EdEmptyLnHldr) {
    $mod = 1;
    $EdEmptyLnHldr = $tEdEmptyLnHldr;
    $varstr =~ s/(\n\$EdEmptyLnHldr\s+\=\s+\').*\'\;/$1$EdEmptyLnHldr\'\;/ || 
      &WebrshDie("$filename appears to be corrupted");
  };
  local($tEdOPFL) = $input{'opfl'};
  if (!$tEdOPFL) {$tEdOPFL = 0;};
  if ($tEdOPFL ne $EdOPFL) {
    $mod = 1;
    $EdOPFL = $tEdOPFL;
    $varstr =~ s/(\n\$EdOPFL\s+\=\s+)\d\;/$1$EdOPFL\;/ || 
      &WebrshDie("$filename appears to be corrupted");
  };
  local($tEdHtmlText) = $input{'htmtxt'};
  if (!$tEdHtmlText) {$tEdHtmlText = 0;};
  if ($tEdHtmlText ne $EdHtmlText) {
    $mod = 1;
    $EdHtmlText = $tEdHtmlText;
    $varstr =~ s/(\n\$EdHtmlText\s+\=\s+)\d\;/$1$EdHtmlText\;/ || 
      &WebrshDie("$filename appears to be corrupted");
  };
  local($tEdTAMod) = $input{'tamod'};
  if ($tEdTAMod) {
    $tEdTAMod =~ m/\'(.*)\'/ || 
      &WebrshDie("Bad \&lt;\/TEXTAREA\&gt; Modifier");
    $tEdTAMod = $1;
  };
  if (!$tEdTAMod) {$tEdTAMod = '';};
  if ($tEdTAMod ne $EdTAMod) {
    $mod = 1;
    $EdTAMod = $tEdTAMod;
    $varstr =~ s/(\n\$EdTAMod\s+\=\s+\').*\'\;/$1$EdTAMod\'\;/ || 
      &WebrshDie("$filename appears to be corrupted");
  };
  return ($varstr,$mod);
}

sub ModGenOptions { # Modifies general preferences.

  local($filename,$varstr) = @_;
  local($mod) = 0;
  local($tBGColor) = $input{'bgcolor'};
  if ($tBGColor ne $BGColor) {
    $mod = 1;
    $BGColor = $tBGColor;
    $varstr =~ s/(\n\$BGColor\s+\=\s+\').*\'\;/$1$BGColor\'\;/ || 
      &WebrshDie("$filename appears to be corrupted");
  };
  local($tBodyParam) = $input{'bparam'};
  if ($tBodyParam ne $BodyParam) {
    $mod = 1;
    $BodyParam = $tBodyParam;
    $tBodyParam =~ s/\'/\\\'/g;
    $varstr =~ s/(\n\$BodyParam\s+\=\s+\').*\'\;/$1$tBodyParam\'\;/ || 
      &WebrshDie("$filename appears to be corrupted");
  };
  local($tTableColor) = $input{'tcolor'};
  if ($tTableColor ne $TableColor) {
    $mod = 1;
    $TableColor = $tTableColor;
    $varstr =~ s/(\n\$TableColor\s+\=\s+\').*\'\;/$1$TableColor\'\;/ || 
      &WebrshDie("$filename appears to be corrupted");
  };
  local($tFTableParam) = $input{'ftparam'};
  if ($tFTableParam ne $FTableParam) {
    $mod = 1;
    $FTableParam = $tFTableParam;
    $tFTableParam =~ s/\'/\\\'/g;
    $varstr =~ s/(\n\$FTableParam\s+\=\s+\').*\'\;/$1$tFTableParam\'\;/ || 
      &WebrshDie("$filename appears to be corrupted");
  };
  local($tTableParam) = $input{'tparam'};
  if ($tTableParam ne $TableParam) {
    $mod = 1;
    $TableParam = $tTableParam;
    $tTableParam =~ s/\'/\\\'/g;
    $varstr =~ s/(\n\$TableParam\s+\=\s+\').*\'\;/$1$tTableParam\'\;/ || 
      &WebrshDie("$filename appears to be corrupted");
  };
  local($tHostName) = $input{'hname'};
  if ($tHostName ne $HostName) {
    $mod = 1;
    $HostName = $tHostName;
    $varstr =~ s/(\n\$HostName\s+\=\s+\').*\'\;/$1$HostName\'\;/ || 
      &WebrshDie("$filename appears to be corrupted");
  };
  local($tComFieldSize) = $input{'cfsize'};
  if ($tComFieldSize ne $ComFieldSize) {
    $mod = 1;
    $ComFieldSize = $tComFieldSize;
    $varstr =~ s/(\n\$ComFieldSize\s+\=\s+\')\d*\'\;/$1$ComFieldSize\'\;/ || 
      &WebrshDie("$filename appears to be corrupted");
  };
  local($tFNFieldSize) = $input{'fnfsize'};
  if ($tFNFieldSize ne $FNFieldSize) {
    $mod = 1;
    $FNFieldSize = $tFNFieldSize;
    $varstr =~ s/(\n\$FNFieldSize\s+\=\s+\')\d*\'\;/$1$FNFieldSize\'\;/ || 
      &WebrshDie("$filename appears to be corrupted");
  };
  local($tFMSize) = $input{'fmsize'};
  if ($tFMSize ne $FMSize) {
    $mod = 1;
    $FMSize = $tFMSize;
    $varstr =~ s/(\n\$FMSize\s+\=\s+\')\d*\'\;/$1$FMSize\'\;/ || 
      &WebrshDie("$filename appears to be corrupted");
  };
  local($tFMHSize) = $input{'fmhsize'};
  if ($tFMHSize < 6) {$tFMHSize = 6;};
  if ($tFMHSize ne $FMHSize) {
    $mod = 1;
    $FMHSize = $tFMHSize;
    $varstr =~ s/(\n\$FMHSize\s+\=\s+\')\d*\'\;/$1$FMHSize\'\;/ || 
      &WebrshDie("$filename appears to be corrupted");
  };
  local($tAppendToPI) = $input{'atpi'};
  $tAppendToPI = 0 if (!$tAppendToPI);
  if ($tAppendToPI ne $AppendToPI) {
    $mod = 1;
    $AppendToPI = $tAppendToPI;
    $varstr =~ s/(\n\$AppendToPI\s+\=\s+)\d\;/$1$AppendToPI\;/ || 
      &WebrshDie("$filename appears to be corrupted");
  };
  local($gob,$got,$tGetOptions) = ($input{'gob'},$input{'got'},'');
  if ($gob) {$tGetOptions = 'B';};
  if ($got) {$tGetOptions = join('',$tGetOptions,'T');};
  if ($tGetOptions ne $GetOptions) {
    $mod = 1;
    $GetOptions = $tGetOptions;
    $varstr =~ s/(\n\$GetOptions\s+\=\s+\').*\'\;/$1$GetOptions\'\;/ || 
      &WebrshDie("$filename appears to be corrupted");
  };
  local($tBinText) = $input{'bintext'};
  $tBinText = 0 if (!$tBinText);
  if ($tBinText ne $BinText) {
    $mod = 1;
    $BinText = $tBinText;
    $varstr =~ s/(\n\$BinText\s+\=\s+)\d\;/$1$BinText\;/ || 
      &WebrshDie("$filename appears to be corrupted");
  };
  local($tDefMimeType) = $input{'dmtype'};
  if ($tDefMimeType ne $DefMimeType) {
    $mod = 1;
    $DefMimeType = $tDefMimeType;
    $varstr =~ s/(\n\$DefMimeType\s+\=\s+\').*\'\;/$1$DefMimeType\'\;/ || 
      &WebrshDie("$filename appears to be corrupted");
  };
  local($tXPath) = $input{'xpath'};
  $tXPath = 0 if (!$tXPath);
  if ($tXPath ne $XPath) {
    $mod = 1;
    $XPath = $tXPath;
    $varstr =~ s/(\n\$XPath\s+\=\s+)\d\;/$1$XPath\;/ || 
      &WebrshDie("$filename appears to be corrupted");
  };
  local($tGCLH) = $input{'gclh'};
  $tGCLH = 0 if (!$tGCLH);
  if ($tGCLH ne $GCLH) {
    $mod = 1;
    $GCLH = $tGCLH;
    $varstr =~ s/(\n\$GCLH\s+\=\s+)\d\;/$1$GCLH\;/ || 
      &WebrshDie("$filename appears to be corrupted");
  };
  local($tPsLib) = $input{'pslib'};
  if ($tPsLib ne $PsLib) {
    $mod = 1;
    $PsLib = $tPsLib;
    $varstr =~ s/(\n\$PsLib\s+\=\s+\').*\'\;/$1$PsLib\'\;/ || 
      &WebrshDie("$filename appears to be corrupted");
  };
  local($tMakeBack) = $input{'mback'};
  $tMakeBack = 0 if (!$tMakeBack);
  if ($tMakeBack ne $MakeBack) {
    $mod = 1;
    $MakeBack = $tMakeBack;
    $varstr =~ s/(\n\$MakeBack\s+\=\s+)\d\;/$1$MakeBack\;/ || 
      &WebrshDie("$filename appears to be corrupted");
  };
  local($tReqMeth) = $input{'rmeth'};
  if ($tReqMeth ne $ReqMeth) {
    $mod = 1;
    $ReqMeth = $tReqMeth;
    $varstr =~ s/(\n\$ReqMeth\s+\=\s+\').*\'\;/$1$ReqMeth\'\;/ || 
      &WebrshDie("$filename appears to be corrupted");
  };
#  local($tBufSize) = $input{'bsize'};
#  if ($tBufSize ne $BufSize) {
#    $mod = 1;
#    $BufSize = $tBufSize;
#    $varstr =~ s/(\n\$BufSize\s+\=\s+)\d+\;/$1$BufSize\;/ || 
#      &WebrshDie("$filename appears to be corrupted");
#  };
  if ($Win32) {
    local($tUseWin32Console) = $input{'uw32c'};
    $tUseWin32Console = 0 if (!$tUseWin32Console);
    if ($tUseWin32Console ne $UseWin32Console) {
      $mod = 1;
      $UseWin32Console = $tUseWin32Console;
      $varstr =~ s/(\n\$UseWin32Console\s+\=\s+)\d\;/$1$UseWin32Console\;/ || 
        &WebrshDie("$filename appears to be corrupted");
    };
  };
  if ($OStype eq 'U') {
    local($tLoginShell) = $input{'logsh'};
    $tLoginShell = 0 if (!$tLoginShell);
    if ($tLoginShell ne $LoginShell) {
      $mod = 1;
      $LoginShell = $tLoginShell;
      $varstr =~ s/(\n\$LoginShell\s+\=\s+)\d\;/$1$LoginShell\;/ || 
        &WebrshDie("$filename appears to be corrupted");
    };
    local($tColFilter) = $input{'colf'};
    $tColFilter = 0 if (!$tColFilter);
    if ($tColFilter ne $ColFilter) {
      $mod = 1;
      $ColFilter = $tColFilter;
      $varstr =~ s/(\n\$ColFilter\s+\=\s+)\d\;/$1$ColFilter\;/ || 
        &WebrshDie("$filename appears to be corrupted");
    };
  };
  return  ($varstr,$mod);
}

sub ModAccOptions { # Modifies access control options.

  local($filename,$varstr) = @_;
  local($mod) = 0;
  local($tLocSecurity) = $input{'locsec'};
  if (!$tLocSecurity) {$tLocSecurity = 0;};
  if ($tLocSecurity ne $LocSecurity) {
    $mod = 1;
    $LocSecurity = $tLocSecurity;
    $varstr =~ s/(\n\$LocSecurity\s+\=\s+)\d\;/$1$LocSecurity\;/ || 
      &WebrshDie("$filename appears to be corrupted");
  };
  local($tAllowAddr) = &PatternMask($input{'allowaddr'});
  local(@tAllowAddr) = &ArrayText($tAllowAddr);
  if (join(':',@tAllowAddr) ne join(':',@AllowAddr)) {
    $mod = 1;
    @AllowAddr = @tAllowAddr;
    local($AllowAddrStr) = &SyntArray(@AllowAddr);
    $varstr =~ s/(\n\@AllowAddr\s+\=\s+\().*\)\;/$1$AllowAddrStr\)\;/ || 
      &WebrshDie("$filename appears to be corrupted");
  };
  local($tDenyAddr) = &PatternMask($input{'denyaddr'});
  local(@tDenyAddr) = &ArrayText($tDenyAddr);
  if (join(':',@tDenyAddr) ne join(':',@DenyAddr)) {
    $mod = 1;
    @DenyAddr = @tDenyAddr;
    local($DenyAddrStr) = &SyntArray(@DenyAddr);
    $varstr =~ s/(\n\@DenyAddr\s+\=\s+\().*\)\;/$1$DenyAddrStr\)\;/ || 
      &WebrshDie("$filename appears to be corrupted");
  };
  local($tCheckForAuth) = $input{'authchk'};
  if (!$tCheckForAuth) {$tCheckForAuth = 0;};
  if ($tCheckForAuth ne $CheckForAuth) {
    $mod = 1;
    $CheckForAuth = $tCheckForAuth;
    $varstr =~ s/(\n\$CheckForAuth\s+\=\s+)\d\;/$1$CheckForAuth\;/ || 
      &WebrshDie("$filename appears to be corrupted");
  };
  local($tCheckForUser) = $input{'usrchk'};
  if (!$tCheckForUser) {$tCheckForUser = 0;};
  if ($tCheckForUser ne $CheckForUser) {
    $mod = 1;
    $CheckForUser = $tCheckForUser;
    $varstr =~ s/(\n\$CheckForUser\s+\=\s+)\d\;/$1$CheckForUser\;/ || 
      &WebrshDie("$filename appears to be corrupted");
  };
  local($tAllowUsers) = $input{'allowusers'};
  local(@tAllowUsers) = &ArrayText($tAllowUsers);
  if (join(':',@tAllowUsers) ne join(':',@AllowUsers)) {
    $mod = 1;
    @AllowUsers = @tAllowUsers;
    local($AllowUsersStr) = &SyntArray(@AllowUsers);
    $varstr =~ s/(\n\@AllowUsers\s+\=\s+\().*\)\;/$1$AllowUsersStr\)\;/ || 
      &WebrshDie("$filename appears to be corrupted");
  };
  return  ($varstr,$mod);
}

sub ModDirOptions { # Modifies directory options.

  local($filename,$varstr) = @_;
  local($mod) = 0;
  local($tProgDir) = $input{'progdir'};
  $tProgDir =~ s/[\\\/]$//;
  if ($tProgDir ne $ProgDir) {
    $mod = 1;
    $ProgDir = $tProgDir;
    $varstr =~ s/(\n\$ProgDir\s+\=\s+\').*\'\;/$1$ProgDir\'\;/ || 
      &WebrshDie("$filename appears to be corrupted");
  };
  $ProgDirPat = $ProgDir; $ProgDirPat =~ s/(\W)/\\$1/g;
  local($tSendFilesDir) = $input{'sfdir'};
  $tSendFilesDir =~ s/[\\\/]$//;
  $tSendFilesDir = &PartPath($tSendFilesDir); 
  if ($tSendFilesDir ne &PartPath($SendFilesDir)) {
    $mod = 1;
    if ($tSendFilesDir =~ m/^(\w\:|\/)/) {
      $SendFilesDir = $tSendFilesDir;
      $varstr =~ s/(\n\$SendFilesDir\s+\=\s+).*\;/$1\'$SendFilesDir\'\;/ || 
        &WebrshDie("$filename appears to be corrupted");
    } else {
      $SendFilesDir = "$ProgDir/$tSendFilesDir";
      $tSendFilesDir =~ s/(\W)/\\$1/g;
      $varstr =~ s/(\n\$SendFilesDir\s+\=\s+).*\;/$1\"\$ProgDir\/$tSendFilesDir\"\;/ || 
        &WebrshDie("$filename appears to be corrupted");
    };
  };
  local($tSendFilesUrl) = $input{'sfurl'};
  $tSendFilesUrl =~ s/^\\//; $tSendFilesUrl =~ s/[\\\/]$//;
#  $tSendFilesUrl =~ s/^([^\/])/\/$1/;
  if ($tSendFilesUrl ne $SendFilesUrl) {
    $mod = 1;
    $SendFilesUrl = $tSendFilesUrl;
    $varstr =~ s/(\n\$SendFilesUrl\s+\=\s+\').*\'\;/$1$SendFilesUrl\'\;/ || 
      &WebrshDie("$filename appears to be corrupted");
  };
  local($tHomeDir) = $input{'homedir'};
  $tHomeDir =~ s/[\\\/]$//;
  $tHomeDir = &PartPath($tHomeDir); 
  if ($tHomeDir ne &PartPath($HomeDir)) {
    $mod = 1;
    if ($tHomeDir =~ m/^(\w\:|\/)/) {
      $HomeDir = $tHomeDir;
      $varstr =~ s/(\n\$HomeDir\s+\=\s+).*\;/$1\'$HomeDir\'\;/ || 
        &WebrshDie("$filename appears to be corrupted");
    } else {
      $HomeDir = "$ProgDir/$tHomeDir";
      $tHomeDir =~ s/(\W)/\\$1/g;
      $varstr =~ s/(\n\$HomeDir\s+\=\s+).*\;/$1\"\$ProgDir\/$tHomeDir\"\;/ || 
        &WebrshDie("$filename appears to be corrupted");
    };
  };
  local($tCfgDir) = $input{'cfgdir'};
  $tCfgDir =~ s/[\\\/]$//;
  $tCfgDir = &PartPath($tCfgDir); 
  if ($tCfgDir ne &PartPath($CfgDir)) {
    $mod = 1;
    if ($tCfgDir =~ m/^(\w\:|\/)/) {
      $CfgDir = $tCfgDir;
      $varstr =~ s/(\n\$CfgDir\s+\=\s+).*\;/$1\'$CfgDir\'\;/ || 
        &WebrshDie("$filename appears to be corrupted");
    } else {
      $CfgDir = "$ProgDir/$tCfgDir";
      $tCfgDir =~ s/(\W)/\\$1/g;
      $varstr =~ s/(\n\$CfgDir\s+\=\s+).*\;/$1\"\$ProgDir\/$tCfgDir\"\;/ || 
        &WebrshDie("$filename appears to be corrupted");
    };
  };
  local($tTempDir) = $input{'tempdir'};
  $tTempDir =~ s/[\\\/]$//;
  $tTempDir = &PartPath($tTempDir); 
  if ($tTempDir ne &PartPath($TempDir)) {
    $mod = 1;
    if ($tTempDir =~ m/^(\w\:|\/)/) {
      $TempDir = $tTempDir;
      $varstr =~ s/(\n\$TempDir\s+\=\s+).*\;/$1\'$TempDir\'\;/ || 
        &WebrshDie("$filename appears to be corrupted");
    } else {
      $TempDir = "$ProgDir/$tTempDir";
      $tTempDir =~ s/(\W)/\\$1/g;
      $varstr =~ s/(\n\$TempDir\s+\=\s+).*\;/$1\"\$ProgDir\/$tTempDir\"\;/ || 
        &WebrshDie("$filename appears to be corrupted");
    };
  };
  local($tPlugInDir) = $input{'pidir'};
  $tPlugInDir =~ s/[\\\/]$//;
  $tPlugInDir = &PartPath($tPlugInDir); 
  if ($tPlugInDir ne &PartPath($PlugInDir)) {
    $mod = 1;
    if ($tPlugInDir =~ m/^(\w\:|\/)/) {
      $PlugInDir = $tPlugInDir;
      $varstr =~ s/(\n\$PlugInDir\s+\=\s+).*\;/$1\'$PlugInDir\'\;/ || 
        &WebrshDie("$filename appears to be corrupted");
    } else {
      $PlugInDir = "$ProgDir/$tPlugInDir";
      $tPlugInDir =~ s/(\W)/\\$1/g;
      $varstr =~ s/(\n\$PlugInDir\s+\=\s+).*\;/$1\"\$ProgDir\/$tPlugInDir\"\;/ || 
        &WebrshDie("$filename appears to be corrupted");
    };
  };
  return  ($varstr,$mod);
}

sub ModMime { # Saves the updated MIME table.

  local($mtable) = @_;
  $mtable =~ s/\r//g;
  open(MTABLE,">$CfgDir/mimetable.txt") || &WebrshDie("Can\'t write MIME table: $!");
  print MTABLE $mtable;
  close(MTABLE);
}

sub TextArray { # Converts an array into a multiline text string.

  return join("\n",@_);
}

sub ArrayText { # Converts a multiline text string to an array.

  local($string) = @_;
  local(@Array) = split("\n",$string);
  local($i) = 0;
  foreach $i (0..$#Array) {
    $Array[$i] =~ s/\r//g; $Array[$i] =~ s/^\s+//; $Array[$i] =~ s/\s+$//;
  };
  @Array = grep $_, @Array;
  return @Array;
}

sub SyntArray { # Converts an array to a string which represents it in Perl syntax.

  local(@Array) = @_;
  foreach $i (0..$#Array) {
    $Array[$i] = join("\'",'',$Array[$i],'');
  };
  local($outstring) = join("\,",@Array);
#  $outstring = join('','(',$outstring,')');
  return $outstring;
}

sub DisplayMask { # Converts a string of regular expression patterns to displayable IP masks.

  local($masks) = @_;
  $masks =~ s/\\d\*/\*/g;
  $masks =~ s/\\d/\?/g;
  $masks =~ s/\\\./\./g;
  return $masks;
}

sub PatternMask { # Converts a string of displayable IP masks to regular expression patterns.

  local($masks) = @_;
  $masks =~ s/\./\\\./g;
  $masks =~ s/\?/\\d/g;
  $masks =~ s/\*/\\d\*/g;
  return $masks;
}

sub PartPath { # Removes $ProgDir from the full path of $ProgDir subdirectories (does not
               # modify any other directory paths). Returns the (possibly) modified path.
               # Requires $Progdir's pattern to reside in $ProgDirPat.

  local($path) = @_;
  $path =~ s/^$ProgDirPat[\\\/]//;
  return $path;
}

sub GetHelp { # Gets the content of an htm help file.

  local($htmfile,$title) = @_;
  local($content) = &ParseHTM(&GetHTM($htmfile));
  $content =~ s/^[\x00-\xff]*\<HR\>//;
  $title = "<B> $title </B>\n"  if ($title);
  $content = join('',$title,'<HR>',$content);
  return $content;
}

sub PrintOptions { # Prints option setting output.

  if ($OpType && $PrOpMenu) {
    if ($OpType eq 'general') {
      &PrintGen;
    } elsif ($OpType eq 'access') {
      &PrintAccess;
    } elsif ($OpType eq 'dirs') {
      &PrintDirs;
    } elsif ($OpType eq 'mime') {
      &PrintMime;
    };
  } else {
    &PrintOpMain;
  };
}

sub PrintOpMain { # Prints the main options menu.

  print '<HTML><HEAD><TITLE> WebRSH - Options </TITLE></HEAD>',"\n",
  '<BODY bgcolor="',"$BGColor\" $BodyParam",'><B> WebRSH: Options: </B>',"\n";
#  if ($!) {print '<H2> The system reported an error: ',"$!",'</H2>',"\n";};
  unless ($output) {
    print '<H4> Your command did not generate any output </H4>',"\n";
  } else {
    if ($output ne 'ok') {print ' <HR><PRE>',&HtmlText($output),'</PRE>',"\n";};
    print '<HR><TABLE><TR><TD><B>',"\n",
    '<A HREF="',"$ScriptUrl",'?id=',"$id",'&pwd=',"$lpwd",'&com=rshoptions&opt=general">',"\n",
    'General Preferences </A></B></TD></TR><TR><TD><B>',"\n",
    '<A HREF="',"$ScriptUrl",'?id=',"$id",'&pwd=',"$lpwd",'&com=rshoptions&opt=access">',"\n",
    'Access Control </A></B></TD></TR><TR><TD><B>',"\n",
    '<A HREF="',"$ScriptUrl",'?id=',"$id",'&pwd=',"$lpwd",'&com=rshoptions&opt=dirs">',"\n",
    'Directories </A></B></TD></TR><TR><TD><B>',"\n",
    '<A HREF="',"$ScriptUrl",'?id=',"$id",'&pwd=',"$lpwd",'&com=rshoptions&opt=mime">',"\n",
    'MIME Table </A></B></TD></TR>',"\n",
    '</TABLE><BR>',"\n";
  };
}

sub PrintAccess { # Prints the access control options menu.

  print '<HTML><HEAD><TITLE> WebRSH - Access Control </TITLE></HEAD>',"\n",
  '<BODY bgcolor="',"$BGColor\" $BodyParam",'><B> WebRSH: Options: Access Control: </B>',"\n";
#  if ($!) {print '<H2> The system reported an error: ',"$!",'</H2>',"\n";};
  unless ($output) {
    print '<H4> Your command did not generate any output </H4>',"\n";
  } else {
    if ($output ne 'ok') {print ' <HR><PRE>',&HtmlText($output),'</PRE>',"\n";};
    print &GetHelp('actrlhlp.htm','Help:') if ($PrintHelp);
    print '<HR>',"\n",
    '<FORM METHOD="',"$ReqMeth",'" ACTION="',"$ScriptUrl",'">',"\n",
    '<INPUT TYPE=hidden name="id" value="',"$id",'">',"\n",
    '<INPUT TYPE=hidden name="pwd" value="',"$pwd",'">',"\n",
    '<INPUT TYPE=hidden name="com" value="rshoptions">',"\n",
    '<INPUT TYPE=hidden name="opt" value="access">',"\n",
    '<INPUT TYPE=checkbox name="locsec" value="1"'; if ($LocSecurity) {print ' CHECKED';}; 
    print '> Restrict Access by Location <BR><BR>',"\n",
    '<TABLE><TR><TD> Allowed Hosts (IP MASKS): <BR>',"\n",
    '<TEXTAREA name="allowaddr" cols=24 rows=8>',&DisplayMask(&TextArray(@AllowAddr)),'</TEXTAREA>',"\n",
    '</TD><TD NOWRAP COLSPAN=12></TD><TD> Denied Hosts (IP MASKS): <BR>',"\n",
    '<TEXTAREA name="denyaddr" cols=24 rows=8>',&DisplayMask(&TextArray(@DenyAddr)),'</TEXTAREA>',"\n",
    '</TD></TR></TABLE><BR>',"\n",
    'Note: The above hosts lists should include precisely one IP mask per line.',"\n",
    'They will be used if and only if the \'Restrict Access by Location\' box is checked.',"\n",
    '<BR><BR><BR>',"\n",
    '<INPUT TYPE=checkbox name="authchk" value="1"'; if ($CheckForAuth) {print ' CHECKED';}; 
    print '> Check for User Authentication <BR><BR><BR>',"\n",
    '<INPUT TYPE=checkbox name="usrchk" value="1"'; if ($CheckForUser) {print ' CHECKED';}; 
    print '> Allow Access Only to these Users: <BR>',"\n",
    '<TEXTAREA name="allowusers" cols=32 rows=8>',&TextArray(@AllowUsers),'</TEXTAREA>',"\n",
    '<BR><BR> Note: The above user list should include precisely one user name per line. It',"\n",
    'will be used if and only if the \'Allow Access Only to these Users\' box is checked.',"\n",
    '<BR><BR><BR><CENTER><TABLE><TR><TD>',"\n",
    '<INPUT TYPE=submit name="opcom" value="    OK    "></TD>',"\n",
    '<TD NOWRAP COLSPAN=2></TD><TD>',"\n",
    '<INPUT TYPE=submit name="opcom" value="  Cancel  "></TD>',"\n",
    '<TD NOWRAP COLSPAN=2></TD><TD>',"\n",
    '<INPUT TYPE=submit name="opcom" value="  Apply   "></TD>',"\n",
    '<TD NOWRAP COLSPAN=2></TD><TD>',"\n",
    '<INPUT TYPE=reset value="  Reset   "></TD>',"\n",
    '<TD NOWRAP COLSPAN=2></TD><TD>',"\n",
    '<INPUT TYPE=submit name="opcom" value="   Help   "></TD>',"\n",
    '</TR></TABLE></CENTER></FORM>',"\n";
  };
}

sub PrintGen { # Prints the general preferences menu

  print '<HTML><HEAD><TITLE> WebRSH - General Preferences </TITLE></HEAD>',"\n",
  '<BODY bgcolor="',"$BGColor\" $BodyParam",'><B> WebRSH: Options: General Preferences: </B>',"\n";
#  if ($!) {print '<H2> The system reported an error: ',"$!",'</H2>',"\n";};
  unless ($output) {
    print '<H4> Your command did not generate any output </H4>',"\n";
  } else {
    print '<HR><PRE>',&HtmlText($output),'</PRE>',"\n" if ($output ne 'ok');
    print &GetHelp('gprfshlp.htm','Help:') if ($PrintHelp);
    $BodyParam =~ s/ onLoad\=\"parent\.UpdateWebRSH\(\)\"$//;
    print '<HR>',"\n",
    '<FORM METHOD="',"$ReqMeth",'" ACTION="',"$ScriptUrl",'">',"\n",
    '<INPUT TYPE=hidden name="id" value="',"$id",'">',"\n",
    '<INPUT TYPE=hidden name="pwd" value="',"$pwd",'">',"\n",
    '<INPUT TYPE=hidden name="com" value="rshoptions">',"\n",
    '<INPUT TYPE=hidden name="opt" value="general">',"\n",
    '<TABLE><TR><TD NOWRAP><B> Background Color: </B></TD><TD NOWRAP COLSPAN=6></TD>',"\n",
    '<TD NOWRAP><INPUT name="bgcolor" value="',"$BGColor",'"></TD></TR></TABLE><BR>',"\n",
    '<TABLE><TR><TD NOWRAP><B> Additional &lt;BODY&gt; Parameters: </B></TD><TD NOWRAP COLSPAN=6></TD>',"\n",
    '<TD NOWRAP><INPUT SIZE="40" name="bparam" value="',&HtmlText($BodyParam),'"></TD></TR></TABLE><BR>',"\n",
    '<TABLE><TR><TD NOWRAP><B> Framing Table Color: </B></TD><TD NOWRAP COLSPAN=6></TD>',"\n",
    '<TD NOWRAP><INPUT name="tcolor" value="',"$TableColor",'"></TD></TR></TABLE><BR>',"\n",
    '<TABLE><TR><TD NOWRAP><B> Additional Framing Table Parameters: </B></TD><TD NOWRAP COLSPAN=6></TD>',"\n",
    '<TD NOWRAP><INPUT SIZE="40" name="ftparam" value="',&HtmlText($FTableParam),'"></TD></TR></TABLE><BR>',"\n",
    '<TABLE><TR><TD NOWRAP><B> Primary Table Parameters: </B></TD><TD NOWRAP COLSPAN=6></TD>',"\n",
    '<TD NOWRAP><INPUT SIZE="40" name="tparam" value="',&HtmlText($TableParam),'"></TD></TR></TABLE><BR>',"\n",
    '<TABLE><TR><TD NOWRAP><B> Host Name: </B></TD><TD NOWRAP COLSPAN=6></TD>',"\n",
    '<TD NOWRAP><INPUT name="hname" value="',"$HostName",'"></TD></TR></TABLE><BR>',"\n",
    '<TABLE><TR><TD NOWRAP><B> Command Field Size: </B></TD><TD NOWRAP COLSPAN=6></TD>',"\n",
    '<TD NOWRAP><INPUT name="cfsize" value="',"$ComFieldSize",'"></TD></TR></TABLE><BR>',"\n",
    '<TABLE><TR><TD NOWRAP><B> FileName Field Size: </B></TD><TD NOWRAP COLSPAN=6></TD>',"\n",
    '<TD NOWRAP><INPUT name="fnfsize" value="',"$FNFieldSize",'"></TD></TR></TABLE><BR>',"\n",
    '<TABLE><TR><TD NOWRAP><B> File Manager Window Size: </B></TD><TD NOWRAP COLSPAN=6></TD>',"\n",
    '<TD NOWRAP> Vertical: <INPUT name="fmsize" size="4" value="',"$FMSize",'"></TD>',"\n",
    '<TD NOWRAP COLSPAN=12></TD><TD NOWRAP> Horizontal: ',"\n",
    '<INPUT name="fmhsize" size="4" value="',"$FMHSize",'"></TD></TR></TABLE><BR>',"\n",
    '<TABLE><TR><TD NOWRAP><B> Plugin Options: </B></TD><TD NOWRAP COLSPAN=6></TD>',"\n",
    '<TD NOWRAP><INPUT TYPE=checkbox name="atpi" value="1"';
    if ($AppendToPI) {print ' CHECKED';}; 
    print '> Append Main Form to Plugins </TD></TR></TABLE><BR>',"\n",
    '<TABLE><TR><TD NOWRAP><B> Optional Downloadable Links: </B></TD>',"\n",
    '<TD NOWRAP COLSPAN=6></TD><TD NOWRAP><INPUT TYPE=checkbox name="got" value="1"';
    if ($GetOptions =~ m/T/) {print ' CHECKED';}; 
    print '> GET as TEXT </TD><TD NOWRAP COLSPAN=12>',"\n",
    '</TD><TD NOWRAP><INPUT TYPE=checkbox name="gob" value="1"';
    if ($GetOptions =~ m/B/) {print ' CHECKED';}; 
    print '> GET as BIN </TD></TR></TABLE><BR>',"\n",
    '<TABLE><TR><TD NOWRAP><B> "GET as TEXT" Sending Mode: </B></TD>',"\n",
    '<TD NOWRAP COLSPAN=6></TD><TD NOWRAP><INPUT TYPE=checkbox name="bintext" value="1"';
    if ($BinText) {print ' CHECKED';}; 
    print '> Send Files in Binary Mode </TD></TR></TABLE><BR>',"\n",
    '<TABLE><TR><TD NOWRAP><B> Default MIME type: </B></TD><TD NOWRAP COLSPAN=6></TD>',"\n",
    '<TD NOWRAP><INPUT SIZE="40" name="dmtype" value="',"$DefMimeType",'"></TD></TR></TABLE><BR>',"\n",
    '<TABLE><TR><TD NOWRAP><B> Extra Path Information: </B></TD>',"\n",
    '<TD NOWRAP COLSPAN=6></TD><TD NOWRAP><INPUT TYPE=checkbox name="xpath" value="1"';
    if ($XPath) {print ' CHECKED';}; 
    print '> Provide Filename for Downloaded Files </TD></TR></TABLE><BR>',"\n",
    '<TABLE><TR><TD NOWRAP><B> Content-length Header: </B></TD>',"\n",
    '<TD NOWRAP COLSPAN=6></TD><TD NOWRAP><INPUT TYPE=checkbox name="gclh" value="1"';
    if ($GCLH) {print ' CHECKED';}; 
    print '> Generate Header when Appropriate </TD></TR></TABLE><BR>',"\n",
    '<TABLE><TR><TD NOWRAP><B> PS Library: </B></TD><TD NOWRAP COLSPAN=6></TD>',"\n",
    '<TD NOWRAP><INPUT name="pslib" value="',"$PsLib",'"></TD></TR></TABLE><BR>',"\n",
    '<TABLE><TR><TD NOWRAP><B> Configuration Backup: </B></TD><TD NOWRAP COLSPAN=6></TD>',"\n",
    '<TD NOWRAP><INPUT TYPE=checkbox name="mback" value="1"';
    if ($MakeBack) {print ' CHECKED';}; 
    print '> Backup the Script whenever Options are Modified </TD></TR></TABLE><BR>',"\n",
    '<TABLE><TR><TD NOWRAP><B> Default Request Method: </B></TD><TD NOWRAP COLSPAN=6></TD>',"\n",
    '<TD NOWRAP><INPUT name="rmeth" value="',"$ReqMeth",'"></TD></TR></TABLE><BR>',"\n";
#    '<TABLE><TR><TD NOWRAP><B> Buffer Size: </B></TD><TD NOWRAP COLSPAN=6></TD>',"\n",
#    '<TD NOWRAP><INPUT name="bsize" value="',"$BufSize",'"></TD></TR></TABLE><BR>',"\n";
    if ($Win32) { print
      '<TABLE><TR><TD NOWRAP><B> Win32 Console Mode: </B></TD><TD NOWRAP COLSPAN=6></TD>',"\n",
      '<TD NOWRAP><INPUT TYPE=checkbox name="uw32c" value="1"';
      if ($UseWin32Console) {print ' CHECKED';}; 
      print '> Assume the Availability of a Full Console </TD></TR></TABLE><BR>',"\n";
    };
    if ($OStype eq 'U') { print
      '<TABLE><TR><TD NOWRAP><B> Initialization Options: </B></TD><TD NOWRAP COLSPAN=6></TD>',"\n",
      '<TD NOWRAP><INPUT TYPE=checkbox name="logsh" value="1"';
      if ($LoginShell) {print ' CHECKED';}; 
      print '> Emulate a Login Shell </TD></TR></TABLE><BR>',"\n",
      '<TABLE><TR><TD NOWRAP><B> Command Output Filtering: </B></TD><TD NOWRAP COLSPAN=6></TD>',"\n",
      '<TD NOWRAP><INPUT TYPE=checkbox name="colf" value="1"';
      if ($ColFilter) {print ' CHECKED';}; 
      print '> Use Col Filter </TD></TR></TABLE><BR>',"\n";
    };
    print '<BR><CENTER><TABLE><TR><TD>',"\n",
    '<INPUT TYPE=submit name="opcom" value="    OK    "></TD>',"\n",
    '<TD NOWRAP COLSPAN=2></TD><TD>',"\n",
    '<INPUT TYPE=submit name="opcom" value="  Cancel  "></TD>',"\n",
    '<TD NOWRAP COLSPAN=2></TD><TD>',"\n",
    '<INPUT TYPE=submit name="opcom" value="  Apply   "></TD>',"\n",
    '<TD NOWRAP COLSPAN=2></TD><TD>',"\n",
    '<INPUT TYPE=reset value="  Reset   "></TD>',"\n",
    '<TD NOWRAP COLSPAN=2></TD><TD>',"\n",
    '<INPUT TYPE=submit name="opcom" value="   Help   "></TD>',"\n",
    '</TR></TABLE></CENTER></FORM>',"\n";
  };
}

sub PrintDirs { # Prints directory preferences menu.

  print '<HTML><HEAD><TITLE> WebRSH - Directories </TITLE></HEAD>',"\n",
  '<BODY bgcolor="',"$BGColor\" $BodyParam",'><B> WebRSH: Options: Directories: </B>',"\n";
#  if ($!) {print '<H2> The system reported an error: ',"$!",'</H2>',"\n";};
  unless ($output) {
    print '<H4> Your command did not generate any output </H4>',"\n";
  } else {
    if ($output ne 'ok') {print ' <HR><PRE>',&HtmlText($output),'</PRE>',"\n";};
    print &GetHelp('dirshlp.htm','Help:') if ($PrintHelp);
    $ProgDirPat = $ProgDir; $ProgDirPat =~ s/(\W)/\\$1/g;
    print '<HR>',"\n",
    'Note: Improper changes to this form may disable most of WebRSH. Partial paths',"\n",
    '(directory paths without a leading slash or a drive letter) are interpreted as subdirectories',"\n",
    'of the \'Main Program Directory\'. Click \'Help\' below for more information. <BR>',"\n",
    '<FORM METHOD="',"$ReqMeth",'" ACTION="',"$ScriptUrl",'">',"\n",
    '<INPUT TYPE=hidden name="id" value="',"$id",'">',"\n",
    '<INPUT TYPE=hidden name="pwd" value="',"$pwd",'">',"\n",
    '<INPUT TYPE=hidden name="com" value="rshoptions">',"\n",
    '<INPUT TYPE=hidden name="opt" value="dirs">',"\n",
    '<TABLE><TR><TD NOWRAP><B> Main Program Directory: </B></TD><TD NOWRAP COLSPAN=6></TD>',"\n",
    '<TD NOWRAP><INPUT SIZE="40" name="progdir" value="',"$ProgDir",'"></TD></TR></TABLE><BR>',"\n",
    '<TABLE><TR><TD NOWRAP><B> File Sending Directory: </B></TD><TD NOWRAP COLSPAN=6></TD>',"\n",
    '<TD NOWRAP><INPUT SIZE="40" name="sfdir" value="',&PartPath($SendFilesDir),'"></TD></TR></TABLE><BR>',"\n",
    '<TABLE><TR><TD NOWRAP><B> File Sending URL: </B></TD><TD NOWRAP COLSPAN=6></TD>',"\n",
    '<TD NOWRAP><INPUT SIZE="40" name="sfurl" value="',"$SendFilesUrl",'"></TD></TR></TABLE><BR>',"\n",
    '<TABLE><TR><TD NOWRAP><B> Home Directory: </B></TD><TD NOWRAP COLSPAN=6>',"\n",
    '</TD><TD NOWRAP><INPUT SIZE="40" name="homedir" value="',&PartPath($HomeDir),'"></TD></TR></TABLE><BR>',"\n",
    '<TABLE><TR><TD NOWRAP><B> Configuration Directory: </B></TD><TD NOWRAP COLSPAN=6>',"\n",
    '</TD><TD NOWRAP><INPUT SIZE="40" name="cfgdir" value="',&PartPath($CfgDir),'"></TD></TR></TABLE><BR>',"\n",
    '<TABLE><TR><TD NOWRAP><B> Temporary Directory: </B></TD><TD NOWRAP COLSPAN=6>',"\n",
    '</TD><TD NOWRAP><INPUT SIZE="40" name="tempdir" value="',&PartPath($TempDir),'"></TD></TR></TABLE><BR>',"\n",
    '<TABLE><TR><TD NOWRAP><B> PlugIn Directory: </B></TD><TD NOWRAP COLSPAN=6>',"\n",
    '</TD><TD NOWRAP><INPUT SIZE="40" name="pidir" value="',&PartPath($PlugInDir),'"></TD></TR></TABLE><BR>',"\n",
    '<BR><CENTER><TABLE><TR><TD>',"\n",
    '<INPUT TYPE=submit name="opcom" value="    OK    "></TD>',"\n",
    '<TD NOWRAP COLSPAN=2></TD><TD>',"\n",
    '<INPUT TYPE=submit name="opcom" value="  Cancel  "></TD>',"\n",
    '<TD NOWRAP COLSPAN=2></TD><TD>',"\n",
    '<INPUT TYPE=submit name="opcom" value="  Apply   "></TD>',"\n",
    '<TD NOWRAP COLSPAN=2></TD><TD>',"\n",
    '<INPUT TYPE=reset value="  Reset   "></TD>',"\n",
    '<TD NOWRAP COLSPAN=2></TD><TD>',"\n",
    '<INPUT TYPE=submit name="opcom" value="   Help   "></TD>',"\n",
    '</TR></TABLE></CENTER></FORM>',"\n";
  };
}

sub PrintMime { # Prints the MIME table for editing.

  print '<HTML><HEAD><TITLE> WebRSH - MIME Table </TITLE></HEAD>',"\n",
  '<BODY bgcolor="',"$BGColor\" $BodyParam",'><B> WebRSH: Options: MIME Table: </B>',"\n";
#  if ($!) {print '<H2> The system reported an error: ',"$!",'</H2>',"\n";};
  unless ($output) {
    print '<H4> Your command did not generate any output </H4>',"\n";
  } else {
    local($buf,$mtable) = ('','');
    open(MTABLE,"<$CfgDir/mimetable.txt") || &WebrshDie("Can\'t open MIME table: $!");
    while (read(MTABLE,$buf,$BufSize)) {
      $mtable .= $buf; # join('',$mtable,$buf);
    };
    close(MTABLE);
    if ($output ne 'ok') {print ' <HR><PRE>',&HtmlText($output),'</PRE>',"\n";};
    print &GetHelp('mimehlp.htm','Help:') if ($PrintHelp);
    print '<HR>',"\n",
    'Note: The MIME table should contain one extension per line, followed by (and',"\n",
    'separated by spaces from) the corresponding MIME type. The extensions',"\n",
    '<EM>must</EM> be capitalized. Click \'Help\' below for more information <BR>',"\n",
    '<CENTER><FORM METHOD="POST" ACTION="',"$ScriptUrl",'">',"\n",
    '<INPUT TYPE=hidden name="id" value="',"$id",'">',"\n",
    '<INPUT TYPE=hidden name="pwd" value="',"$pwd",'">',"\n",
    '<INPUT TYPE=hidden name="com" value="rshoptions">',"\n",
    '<INPUT TYPE=hidden name="opt" value="mime">',"\n",
    '<TEXTAREA name="mtable" cols="48" rows="12">',"$mtable",'</TEXTAREA>',"\n",
    '<BR><BR><TABLE><TR><TD>',"\n",
    '<INPUT TYPE=submit name="opcom" value="    OK    "></TD>',"\n",
    '<TD NOWRAP COLSPAN=2></TD><TD>',"\n",
    '<INPUT TYPE=submit name="opcom" value="  Cancel  "></TD>',"\n",
    '<TD NOWRAP COLSPAN=2></TD><TD>',"\n",
    '<INPUT TYPE=submit name="opcom" value="  Apply   "></TD>',"\n",
    '<TD NOWRAP COLSPAN=2></TD><TD>',"\n",
    '<INPUT TYPE=reset value="  Reset   "></TD>',"\n",
    '<TD NOWRAP COLSPAN=2></TD><TD>',"\n",
    '<INPUT TYPE=submit name="opcom" value="   Help   "></TD>',"\n",
    '</TR></TABLE></FORM></CENTER>',"\n";
  };
}

1; # return true
