# pslib-win32.pl - Process handling subroutines for Win32
# This file is part of WebRSH 1.1b (beta)
# Copyright (C) 1997,1998  Yoram Last (ylast@mindless.com)
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#
# These subroutines require an appropriate 'ps.exe' to reside in
# $ProgDir\bin
#

sub RshPs { # Returns a list of currently running processes.

  return &ExWin32Com("$ProgDir\\bin\\ps.exe -w");
}

sub RshKill { # Terminates a process.

  local($pid) = @_;
  kill 9, $pid;
  local($err) = Win32::GetLastError;
  unless ($err) {
    return "Process $pid was terminated\n";
  } else {
    return Win32::FormatMessage $err;
  };
}

sub GetKillList { # Creates three equal-length arrays (@kname,@kpid,@lkill) containing names,
                  # PIDs, and url-encoded kill commands for currently running processes.

  local($pid,$name);
  $killlist = &ExWin32Com("$ProgDir\\bin\\ps.exe");
#  $killlist =~ s/.*\n//;
  $killlist =~ s/\n$//;
  @killarray = split(/\n/,$killlist);
  foreach $i (0..$#killarray) {
    $killarray[$i] =~ s/^\s+(\d)/$1/;
    ($pid,$name) = split(/\s+/,$killarray[$i],2);
    $kpid[$i] = $pid;
    $lkill[$i] = "rshkill $kpid[$i]";
    $lkill[$i] = &UrlEncode($lkill[$i]);
    $kname[$i] = $name;
  };
}

sub PrintKill { # Prints the list of "Killable Links."

print '<HTML><HEAD><TITLE> WebRSH - Killable links </TITLE></HEAD>',"\n",
  '<BODY bgcolor="',"$BGColor\" $BodyParam",'><B> WebRSH: Killable Links: </B>',"\n";
#  if ($!) {print '<H2> The system reported an error: ',"$!",'</H2>',"\n";};
  unless ($output) {print '<P> Your command did not generate any output </P>',"\n";}
  else {
    print '<P> The list below contains links to processes currently running on ',"\n",
    "$HostName",'. Clicking any of these links will TERMINATE (kill) the corresponding process.',"\n",
    '</P><CENTER><HR></CENTER><TABLE>',"\n";
    foreach $i (0..$#kpid) {
      print '<TR><TD NOWRAP><B>',"\n",
      '<A HREF="',"$ScriptUrl",'?id=',"$id",'&pwd=',"$lpwd",'&com=',"$lkill[$i]",'">',"\n",
      '<IMG SRC="',"$SendFilesUrl",'/redball.gif" HSPACE=4 BORDER=0> ! KILL:  ',"$kname[$i]",'</A>',"\n",
      ' [ pid = ',"$kpid[$i]",' ] </B></TD></TR>',"\n";
    };
  };
  print '<TR><TD></TD></TR></TABLE>',"\n";
}

1; # return true